﻿using System;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace DotNet.Common.Util
{
    /// <summary>
    /// 记事本辅助类
    /// </summary>
    public class NotePadUtil
    {

        /// <summary>
        /// 打开记事本
        /// </summary>
        public static void OpenNotePad(ProcessWindowStyle winStyle)
        {
            var info = new ProcessStartInfo
            {
                FileName = "notepad.exe",
                WorkingDirectory = "c:/windows/",

                WindowStyle = winStyle,

                Arguments = string.Empty,
            };

            try
            {
                var proc = Process.Start(info);
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// 打开记事本 并填充内容
        /// </summary>
        /// <param name="winStyle"></param>
        /// <param name="content"></param>
        public static void OpenNotePad(ProcessWindowStyle winStyle, string content)
        {
            content = content ?? string.Empty;

            content = content.Replace("\r", Environment.NewLine).Replace("\n", Environment.NewLine);

            Process proc = null;

            var info = new ProcessStartInfo
            {
                FileName = "notepad.exe",
                WorkingDirectory = "c:/windows/",

                WindowStyle = winStyle,

                Arguments = string.Empty,
            };

            try
            {
                proc = Process.Start(info);
            }
            catch
            {
                proc = null;
            }

            if (proc == null)
            {
                return;
            }


            // 调用 API, 传递数据
            while (proc.MainWindowHandle == IntPtr.Zero)
            {
                proc.Refresh();
            }

            IntPtr vHandle = FindWindowEx(proc.MainWindowHandle, IntPtr.Zero, "Edit", null);

            // 传递数据给记事本
            SendMessage(vHandle, WM_SETTEXT, 0, content);

        }

        #region 记事本辅助

        /// <summary>
        /// 记事本需要的常量
        /// </summary>
        private const uint WM_SETTEXT = 0x000C;

        /// <summary>
        /// 传递消息给记事本
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="Msg"></param>
        /// <param name="wParam"></param>
        /// <param name="lParam"></param>
        /// <returns></returns>
        [DllImport("User32.DLL")]
        private static extern int SendMessage(IntPtr hWnd, uint Msg, int wParam, string lParam);

        /// <summary>
        /// 查找句柄
        /// </summary>
        /// <param name="hwndParent"></param>
        /// <param name="hwndChildAfter"></param>
        /// <param name="lpszClass"></param>
        /// <param name="lpszWindow"></param>
        /// <returns></returns>
        [DllImport("User32.DLL")]
        private static extern IntPtr FindWindowEx(IntPtr hwndParent, IntPtr hwndChildAfter, string lpszClass, string lpszWindow);

        #endregion

    }
}
