﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading;

namespace DotNet.Common.Util
{
    /// <summary>
    /// 记录日志 本地测试查看日志使用 
    /// 
    /// web应用程序通常将日志文件独立保存于一个固定目录下 C:\PowerLogger
    /// </summary>
    public class PowerLogger
    {
        #region 字段

        private static readonly object objSync = new object();

        /// <summary>
        /// 默认一个文本文件的最大值为5M
        /// </summary>
        private static readonly long maxFileLength = 1024 * 1024 * 5;

        #endregion

        #region 方法

        private static void AppendLog(string msg)
        {
            bool isLocked = false;
            try
            {
                var strDateTime = DateTime.Now.ToString("yyyyMMdd");

                isLocked = Monitor.TryEnter(objSync, 100);
                if (isLocked == false)
                {
                    return;
                }

                var logPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory,
                        string.Format("LOG_{0}", DateTime.Now.ToString("MM")));  //保存日志所在路径
                var isWeb = CheckIsWebApp(); //是否web应用
                if (isWeb == true)
                {
                    logPath = string.Format(@"C:\PowerLogger\LOG_{0}", DateTime.Now.ToString("MM"));
                }

                if (Directory.Exists(logPath) == false)
                {
                    Directory.CreateDirectory(logPath);
                }

                logPath = Path.Combine(logPath, strDateTime);
                if (Directory.Exists(logPath) == false)
                {
                    Directory.CreateDirectory(logPath);
                }

                var stackFrame = new StackFrame(2, false);
                logPath = Path.Combine(logPath, stackFrame.GetMethod().DeclaringType.FullName);
                if (Directory.Exists(logPath) == false)
                {
                    Directory.CreateDirectory(logPath);
                }

                var currentIndex = 0;
                currentIndex = Directory.GetFiles(logPath)
                    .Where(x => x.Contains(".log") && x.Contains("_") && new FileInfo(x).Name.Contains(strDateTime))
                    .Select(x =>
                    {
                        var index = -1;
                        int.TryParse(new FileInfo(x).Name.Split('_')[1].Replace(".log", string.Empty), out index);
                        return Math.Abs(index);
                    })
                    .OrderByDescending(x => x).FirstOrDefault();

                string filePath = Path.Combine(logPath, string.Format("{0}_{1}.log", strDateTime, currentIndex));

                if (File.Exists(filePath) == true)
                {
                    var fileInfo = new FileInfo(filePath);
                    var len = fileInfo.Length;
                    //单个文件超过最大值 新建一个文本文件
                    if (len > maxFileLength)
                    {
                        filePath = Path.Combine(logPath, string.Format("{0}_{1}", strDateTime, currentIndex + 1) + ".log");
                    }
                }

                File.AppendAllText(filePath, DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss:fff"), System.Text.Encoding.UTF8);
                File.AppendAllText(filePath, Environment.NewLine);
                File.AppendAllText(filePath, msg, System.Text.Encoding.UTF8);
                File.AppendAllText(filePath, Environment.NewLine, System.Text.Encoding.UTF8);
                File.AppendAllText(filePath, Environment.NewLine, System.Text.Encoding.UTF8);
            }
            catch
            {
            }
            finally
            {
                if (isLocked)
                {
                    Monitor.Exit(objSync);
                }
            }
        }

        public static void AppendLog(object msg)
        {
            if (msg == null)
            {
                return;
            }
            if (msg is string)
            {
                AppendLog(msg);
                return;
            }
            AppendLog(msg.ToString());
        }

        public static void AppendLog(Exception ex)
        {
            if (ex == null)
            {
                return;
            }
            AppendLog(ex.ToString());
        }

        public static void AppendLog(string msg, params object[] objArr)
        {
            try
            {
                if (objArr == null)
                {
                    AppendLog(msg);
                    return;
                }
                string strMsg = string.Format(msg, objArr);
                AppendLog(strMsg);
            }
            catch
            {
            }
        }

        #endregion

        #region 帮助方法

        /// <summary>
        /// 判断当前应用是否为web
        /// </summary>
        /// <returns></returns>
        private static bool CheckIsWebApp()
        {
            var isWeb = false;
            var hsWebProcess = new HashSet<string>
            {
                "w3wp",
                "aspnet_wp",
                "webdev.webserver"
            };

            try
            {
                var processName = Process.GetCurrentProcess().ProcessName.ToLower();
                if (hsWebProcess.Contains(processName) == true)
                {
                    isWeb = true;
                }
                else if (System.Web.HttpContext.Current != null)
                {
                    isWeb = true;
                }
            }
            catch
            {
                isWeb = false;
            }

            return isWeb;
        }

        #endregion

    }
}