﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Web;

namespace DotNet.Common.Util
{
    public class WebRequestUtil
    {
        /// <summary>
        /// GET请求  默认编码：UTF-8  timeOut是超时时间，以秒为单位，默认为30秒
        /// </summary>
        /// <param name="url">远程访问地址 如：http://www.huazhu.com</param>
        /// <param name="sortedList">querystring参数字典 可以为空</param>
        /// <returns></returns>
        public static PowerBizResult<string> HttpGet(string url, SortedList<string, string> sortedList,
            string charset = "UTF-8", int timeOut = 30)
        {
            var resultInfo = new PowerBizResult<string>(true);
            var respMsg = string.Empty;
            var respCode = 0;//响应码

            var encoder = Encoding.UTF8;

            HttpWebResponse response = null;
            try
            {
                if (string.IsNullOrWhiteSpace(charset) == false)
                {
                    encoder = Encoding.GetEncoding(charset);
                }

                var sb = new StringBuilder(256);
                if (sortedList != null)
                {
                    foreach (var kv in sortedList)
                    {
                        sb.AppendFormat("{0}={1}&", kv.Key, HttpUtility.UrlEncode(kv.Value, encoder));
                    }
                }

                var queryString = sb.ToString().Trim('&');
                if (queryString.Length > 0)
                {
                    if (url.LastIndexOf('?') == -1)
                    {
                        url = string.Format("{0}?{1}", url, queryString);
                    }
                    else
                    {
                        url = string.Format("{0}&{1}", url, queryString);
                    }
                }

                var request = (HttpWebRequest)WebRequest.Create(url);

                if (timeOut < 1)
                {
                    timeOut = 30;
                }
                //超时时间,以秒为单位 
                request.Timeout = timeOut * 1000;

                //request.Proxy = new WebProxy(ProxyString, true); //true means no proxy
                ServicePointManager.ServerCertificateValidationCallback = new RemoteCertificateValidationCallback(CheckValidate);

                response = (HttpWebResponse)request.GetResponse();
                var sr = new StreamReader(response.GetResponseStream(), encoder);

                respMsg = sr.ReadToEnd();

                respCode = Convert.ToInt32(response.StatusCode);

                sr.Close();
                response.Close();
            }
            catch (Exception ex)
            {
                resultInfo.IsOK = false;
                if (response != null)
                {
                    respCode = Convert.ToInt32(response.StatusCode);
                }
                respMsg = string.Format("http响应码：{0}======异常信息:{1}", respCode, ex.ToString());
            }

            resultInfo.ReturnObject = respMsg;
            resultInfo.Code = respCode;

            return resultInfo;
        }

        /// <summary>
        /// POST请求  默认编码：UTF-8  timeOut是超时时间，以秒为单位，默认为30秒
        /// </summary>
        /// <param name="url">远程访问地址 如：http://www.huazhu.com</param>
        /// <param name="sortedList">post表单参数字典 可以为空</param>
        /// <returns></returns>
        public static PowerBizResult<string> HttpPost(string url, SortedList<string, string> sortedList,
            string charset = "UTF-8", int timeOut = 30)
        {
            var resultInfo = new PowerBizResult<string>(true);
            var respMsg = string.Empty;
            var respCode = 0;//响应码

            var encoder = Encoding.UTF8;

            HttpWebResponse response = null;
            try
            {
                if (string.IsNullOrWhiteSpace(charset) == false)
                {
                    encoder = Encoding.GetEncoding(charset);
                }

                var request = (HttpWebRequest)WebRequest.Create(url);
                if (timeOut < 1)
                {
                    timeOut = 30;
                }
                //超时时间,以秒为单位 
                request.Timeout = timeOut * 1000;

                //request.Proxy = new WebProxy(ProxyString, true); //true means no proxy
                ServicePointManager.ServerCertificateValidationCallback = new RemoteCertificateValidationCallback(CheckValidate);

                var sb = new StringBuilder(256);
                if (sortedList != null)
                {
                    foreach (var kv in sortedList)
                    {
                        sb.AppendFormat("{0}={1}&", kv.Key, HttpUtility.UrlEncode(kv.Value, encoder));
                    }
                }

                var postData = sb.ToString().Trim('&');

                if (postData.Length > 0)
                {
                    byte[] data = Encoding.UTF8.GetBytes(postData);

                    request.Method = "POST";

                    request.ContentType = "application/x-www-form-urlencoded";

                    request.ContentLength = data.Length;

                    Stream ws = request.GetRequestStream();

                    // 发送数据
                    ws.Write(data, 0, data.Length);
                    ws.Close();
                }

                response = (HttpWebResponse)request.GetResponse();
                var sr = new StreamReader(response.GetResponseStream(), encoder);

                respMsg = sr.ReadToEnd();

                respCode = Convert.ToInt32(response.StatusCode);

                sr.Close();
                response.Close();
            }
            catch (Exception ex)
            {
                resultInfo.IsOK = false;
                if (response != null)
                {
                    respCode = Convert.ToInt32(response.StatusCode);
                }
                respMsg = string.Format("http响应码：{0}======异常信息:{1}", respCode, ex.ToString());
            }

            resultInfo.ReturnObject = respMsg;
            resultInfo.Code = respCode;

            return resultInfo;
        }

        /// <summary>
        /// 验证服务器证书
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="certificate"></param>
        /// <param name="chain"></param>
        /// <param name="errors"></param>
        /// <returns></returns>
        private static bool CheckValidate(object sender,
            X509Certificate certificate, X509Chain chain, SslPolicyErrors errors)
        {
            return true;
        }
    }
}