﻿using System;
using System.Collections.Generic;

namespace PowerServiceMonitor
{
    using PowerServiceMonitor.Contract;
    using PowerServiceMonitor.Impl;
    using PowerServiceMonitor.Model;
    using PowerServiceMonitor.Util;

    /// <summary>
    /// 日志服务工厂
    /// </summary>
    public class LogServiceBuilder
    {
        #region 公共方法

        public static void AppendLog(object msg)
        {
            try
            {
                var dictLogSvc = GetConfigLogService();
                foreach (var kv in dictLogSvc)
                {
                    kv.Value.AppendLog(msg);
                }
            }
            catch
            {
            }
        }

        public static void AppendLog(Exception ex)
        {
            try
            {
                var dictLogSvc = GetConfigLogService();
                foreach (var kv in dictLogSvc)
                {
                    kv.Value.AppendLog(ex);
                }
            }
            catch
            {
            }
        }

        public static void AppendLog(string msg, params object[] objArr)
        {
            try
            {
                var dictLogSvc = GetConfigLogService();
                foreach (var kv in dictLogSvc)
                {
                    kv.Value.AppendLog(msg, objArr);
                }
            }
            catch
            {
            }
        }

        #endregion

        #region 辅助方法

        private static IDictionary<string, ILog> GetAllLogServiceDict()
        {
            var dict = new Dictionary<string, ILog>();

            dict[LogProviderType.Text.ToString().ToLower()] = new TextLogService();
            dict[LogProviderType.DB.ToString().ToLower()] = new DBLogService();
            dict[LogProviderType.NoSQL.ToString().ToLower()] = new NoSQLLogService();

            return dict;
        }

        private static IDictionary<string, ILog> GetConfigLogService()
        {
            var dict = new Dictionary<string, ILog>();

            var allLogDict = GetAllLogServiceDict();

            var logTypeArr = MonitorConfig.GetLogProviderTypeArr();

            foreach (var item in logTypeArr)
            {
                ILog svc = null;
                if (string.IsNullOrWhiteSpace(item))
                {
                    continue;
                }

                allLogDict.TryGetValue(item.ToLower(), out svc);

                if (svc == null)
                {
                    continue;
                }

                dict[item] = svc;
            }

            return dict;
        }

        #endregion
    }

}
