﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;

namespace PowerServiceMonitor
{
    using DotNet.Common.Util;
    using PowerServiceMonitor.Contract;
    using PowerServiceMonitor.Impl;
    using PowerServiceMonitor.Model;
    using PowerServiceMonitor.Util;

    public class ServiceFactory
    {
        private static IDictionary<AppTypeEnum, IServiceMonitor> serviceDict = new Dictionary<AppTypeEnum, IServiceMonitor>();

        static ServiceFactory()
        {
            serviceDict[AppTypeEnum.WebPage] = new WebPageMonitorBuilder();

            serviceDict[AppTypeEnum.WebService] = new WebServiceMonitorBuilder();

            serviceDict[AppTypeEnum.WindowsService] = new WinServiceMonitorBuilder();
        }

        //public static IDictionary<AppTypeEnum, IServiceMonitor> GetMonitorServices()
        //{
        //    return serviceDict;
        //}

        /// <summary>
        /// 输出监控结果
        /// </summary>
        /// <returns></returns>
        public static string OutPutMonitor()
        {
            var stopwatch = Stopwatch.StartNew();

            var sb = new StringBuilder(1024);
            sb.AppendFormat("=============={0}监控启动==============", DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss:fff"));
            sb.AppendLine();

            foreach (var kv in serviceDict)
            {
                sb.AppendFormat("==={0}监控开始===", EnumHelper.GetEnumDescription<AppTypeEnum>(kv.Key));
                sb.AppendLine();

                var monitorResults = kv.Value.Start(MonitorConfig.MonitorConfigFilePath);
                if (monitorResults.Any() == false)
                {
                    sb.Append("没有找到此类应用下需要监控的项目");
                    sb.AppendLine();
                }
                else
                {
                    foreach (var result in monitorResults)
                    {
                        if (result.IsOK == false)
                        {
                            sb.AppendFormat("【警告】【警告】监控：{0} 结果：{1},信息:{2}", result.CurrentMonitorItemInfo.SvcName, result.IsOK, result.Message);
                        }
                        else
                        {
                            sb.AppendFormat("{0}状态正常", result.CurrentMonitorItemInfo.SvcName);
                        }
                        sb.AppendLine();
                    }
                }

                sb.AppendFormat("==={0}监控结束。===", EnumHelper.GetEnumDescription<AppTypeEnum>(kv.Key));
                sb.AppendLine();
                sb.AppendLine();
            }

            stopwatch.Stop();

            sb.AppendFormat("本次监控耗时：{0}小时{1}分{2}秒{3}毫秒",
                stopwatch.Elapsed.Hours, stopwatch.Elapsed.Minutes, stopwatch.Elapsed.Seconds, stopwatch.Elapsed.Milliseconds);

            return sb.ToString();
        }

    }
}
