﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;

namespace PowerServiceMonitor.Impl
{
    using DotNet.Common.Util;
    using PowerServiceMonitor.AgentService;
    using PowerServiceMonitor.Contract;
    using PowerServiceMonitor.Model;
    using PowerServiceMonitor.Util;

    public class WebServiceMonitorBuilder : IServiceMonitor
    {

        #region 接口方法

        public IList<MonitorResultInfo> Start(string monitorConfigPath)
        {
            var monitorResult = new List<MonitorResultInfo>();

            var monitorItems = MonitorItemUtil.LoadMonitorItems(monitorConfigPath)
                .Where(x => string.Compare(x.AppType, AppTypeEnum.WebService.ToString(), true) == 0)
                .Where(x => x.IsEnabled == true)
                .Where(x => string.IsNullOrWhiteSpace(x.SvcValue) == false);

            foreach (var item in monitorItems)
            {
                var resultInfo = AutoMonitor(item);
                monitorResult.Add(resultInfo);

                if (resultInfo.IsOK == true)
                {
                    continue;
                }
                //监控有错误 发送警告提醒
                if (item.IsSendEmail == true)
                {
                    AlertService.SendEmail(resultInfo.Message);
                }
                if (item.IsSendSMS == true)
                {
                    AlertService.SendSMSMsg(resultInfo.Message);
                }
            }

            return monitorResult;
        }

        #endregion

        #region 辅助方法

        /// <summary>
        /// 自动监控 (如第一次访问不成功，重试2次) 如最终返回http状态码不是200而是4或5开头 可能web服务停止或配置不正确
        /// </summary>
        private MonitorResultInfo AutoMonitor(MonitorItemInfo monitorItemInfo)
        {
            var resultInfo = new MonitorResultInfo
            {
                IsOK = true,
                Message = string.Empty,
                CurrentMonitorItemInfo = monitorItemInfo,
            };

            var retHTML = string.Empty;

            try
            {
                var bizResult = WebRequestUtil.HttpGet(monitorItemInfo.SvcValue.Trim(), null);
                if (bizResult.IsOK == false || bizResult.Code != 200)
                {
                    Thread.Sleep(100);
                    bizResult = WebRequestUtil.HttpGet(monitorItemInfo.SvcValue.Trim(), null);
                }
                if (bizResult.IsOK == false || bizResult.Code != 200)
                {
                    Thread.Sleep(100);
                    bizResult = WebRequestUtil.HttpGet(monitorItemInfo.SvcValue.Trim(), null);
                }

                var retryCount = 2; //重试次数

                if (bizResult.IsOK == false || bizResult.Code != 200)
                {
                    for (int i = 0; i < retryCount; i++)
                    {
                        Thread.Sleep(100);
                        bizResult = WebRequestUtil.HttpGet(monitorItemInfo.SvcValue.Trim(), null);
                        if (bizResult.Code == 200)
                        {
                            break;
                        }
                    }
                }

                resultInfo.IsOK = bizResult.IsOK;
                resultInfo.Message = string.Format("守护进程PowerMonitorService工作ing...,Web服务{0},返回状态码:{1}，返回信息:{2}",
                    monitorItemInfo.SvcValue, bizResult.Code, bizResult.Message);

            }
            catch (Exception ex)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = string.Format("Web服务{0}自动监控发生异常:{1}", monitorItemInfo.SvcValue, ex);
            }
            finally
            {
                retHTML = retHTML ?? string.Empty;
                if (retHTML.Length > 1024)
                {
                    retHTML = retHTML.Substring(0, 1024);
                }
                LogServiceBuilder.AppendLog("Web页面{0}自动监控,结果：{1}，信息：{2},响应html:{3}"
                    , monitorItemInfo.SvcValue, resultInfo.IsOK, resultInfo.Message, retHTML);
            }

            return resultInfo;
        }

        #endregion
    }
}
