﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceProcess;

namespace PowerServiceMonitor.Impl
{
    using DotNet.Common.Util;
    using PowerServiceMonitor.AgentService;
    using PowerServiceMonitor.Contract;
    using PowerServiceMonitor.Model;
    using PowerServiceMonitor.Util;

    public class WinServiceMonitorBuilder : IServiceMonitor
    {
        #region 接口方法

        public IList<MonitorResultInfo> Start(string monitorConfigPath)
        {
            var monitorResult = new List<MonitorResultInfo>();

            var monitorItems = MonitorItemUtil.LoadMonitorItems(monitorConfigPath)
                .Where(x => string.Compare(x.AppType, AppTypeEnum.WindowsService.ToString(), true) == 0)
                .Where(x => x.IsEnabled == true)
                .Where(x => string.IsNullOrWhiteSpace(x.SvcValue) == false);

            foreach (var item in monitorItems)
            {
                var resultInfo = AutoMonitor(item);
                monitorResult.Add(resultInfo);

                if (resultInfo.IsOK == true)
                {
                    continue;
                }
                //监控有错误 发送警告提醒
                if (item.IsSendEmail == true)
                {
                    AlertService.SendEmail(resultInfo.Message);
                }
                if (item.IsSendSMS == true)
                {
                    AlertService.SendSMSMsg(resultInfo.Message);
                }
            }

            return monitorResult;
        }

        #endregion

        #region 辅助方法

        /// <summary>
        /// 自动监控 如已经停止，尝试自动启动 防止windows服务意外停止
        /// </summary>
        private MonitorResultInfo AutoMonitor(MonitorItemInfo monitorItemInfo)
        {
            var resultInfo = new MonitorResultInfo
            {
                IsOK = true,
                Message = string.Empty,
                CurrentMonitorItemInfo = monitorItemInfo,
            };

            ServiceController control = null;
            if (WinServiceUtil.CheckWinServiceExists(monitorItemInfo.SvcValue.Trim(), out control) == false) //不存在配置的服务
            {
                resultInfo.IsOK = false;
                resultInfo.Message = string.Format("不存在配置的Windows服务:{0}", monitorItemInfo.SvcValue.Trim());

                return resultInfo;
            }
            try
            {
                if (control.Status == ServiceControllerStatus.Running) //服务运行中 直接返回
                {
                    return resultInfo;
                }

                LogServiceBuilder.AppendLog(string.Format("Windows服务{0}当前状态:{1},服务需要维持的状态应该是:{2}",
                    control.ServiceName, control.Status.ToString(), ServiceControllerStatus.Running.ToString()));

                control.WaitForStatus(ServiceControllerStatus.Running); //最多等待2分钟
                //control.WaitForStatus(ServiceControllerStatus.Running, new TimeSpan(0, 0, 120)); //最多等待2分钟

                resultInfo.Message = string.Format("守护进程PowerMonitorService工作ing...,服务{0}当前状态:{1}",
                    control.ServiceName, ServiceControllerStatus.Running);

            }
            catch (Exception ex)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = string.Format("Windows服务{0}自动监控发生异常:{1}", monitorItemInfo.SvcValue, ex);
            }
            finally
            {
                LogServiceBuilder.AppendLog("Web页面{0}自动监控,结果：{1}，信息：{2}"
                    , monitorItemInfo.SvcValue, resultInfo.IsOK, resultInfo.Message);
            }

            return resultInfo;
        }

        #endregion
    }
}
