﻿using System;
using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ServiceMonitor
{
    using DotNet.Common.Util;
    using PowerServiceMonitor;
    using PowerServiceMonitor.Util;
    using PowerServiceMonitor.WinControl;

    public partial class MainForm : Form
    {
        #region 构造函数

        public MainForm()
        {
            var loginForm = new LogInForm();
            var result = loginForm.ShowDialog();
            if (result != System.Windows.Forms.DialogResult.OK)
            {
                System.Environment.Exit(0);
                return;
            }

            InitializeComponent();

            //loading控件控制
            InitLoadingCtrl();

            InitTimer();

            InitAppSettingConfigInfo();

            this.SizeChanged += new EventHandler(MainForm_SizeChanged);

            ////5分钟后自动隐藏窗体
            //var tskFactory = new TaskFactory();
            //tskFactory.StartNew(() =>
            //{
            //    SetFormHidden();
            //});
        }

        #endregion

        #region 字段

        /// <summary>
        /// 服务监控定时器
        /// </summary>
        private static System.Windows.Forms.Timer currentTimer = null;

        private PowerLoadingLayer loadingCtrl = null;//loading效果控件

        #endregion

        #region 定时器

        public void InitTimer()
        {
            //KeepWatching(null);

            int interval = 1000 * 60 * MonitorConfig.GetIntervalMinutes();//每隔多少分钟监控一次
            currentTimer = new System.Windows.Forms.Timer(); //KeepWatching, null, 1000 * 2, Timeout.Infinite
            currentTimer.Tick += new EventHandler(currentTimer_Tick);
            currentTimer.Interval = interval;
            currentTimer.Start();
        }

        void currentTimer_Tick(object sender, EventArgs e)
        {
            var tskFactory = new TaskFactory();
            tskFactory.StartNew(() =>
            {
                //Thread.Sleep(1000);
                KeepWatching(null);
            });
        }

        #endregion

        #region 方法

        private void InitAppSettingConfigInfo()
        {
            this.cbConfigType.SelectedIndex = 0;
            this.txtConfigKey.Text = MonitorConfig.MonitorInterval;
            this.txtConfigValue.Text = MonitorConfig.GetIntervalMinutes().ToString();
        }

        private void InitLoadingCtrl()
        {
            this.loadingCtrl = new PowerLoadingLayer(125, true);
            this.gbMonitorArea.Controls.Add(this.loadingCtrl);
            this.loadingCtrl.Dock = DockStyle.Fill;
            this.loadingCtrl.BringToFront();
            this.loadingCtrl.Visible = false;
        }

        private void SetFormHidden()
        {
            //Thread.Sleep(1000 * 10); //10秒后自动隐藏

            Thread.Sleep(1000 * 60 * 5); //5分钟自动隐藏

            if (this.InvokeRequired == false)
            {
                this.WindowState = FormWindowState.Minimized;
                this.Hide();
            }
            else
            {
                Action action = SetFormHidden;
                this.BeginInvoke(action, null);
            }
        }

        private void SetOutput(string msg)
        {
            if (!string.IsNullOrWhiteSpace(msg))
            {
                LogServiceBuilder.AppendLog(msg);
            }
            if (this.txtMsg.InvokeRequired == false)
            {
                if (string.IsNullOrEmpty(msg) == true)
                {
                    this.txtMsg.Clear();
                }

                var maxLen = 100000;//文本框显示的最大字符数 默认10万字符
                if (this.txtMsg.Text.Length > maxLen)
                {
                    this.txtMsg.Text = this.txtMsg.Text.Substring(0, maxLen); //截取一部分显示
                    //this.txtMsg.Clear();
                }

                this.txtMsg.Text += msg;
                if (string.IsNullOrEmpty(msg) == false)
                {
                    this.txtMsg.Text += Environment.NewLine;
                }
            }
            else
            {
                Action<string> action = SetOutput;
                this.BeginInvoke(action, new object[] { msg });
            }
        }

        /// <summary>
        /// 保持监控
        /// </summary>
        private void KeepWatching(object state)
        {
            SetOutput(null);

            try
            {
                this.ShowLoadingLayer();//显示loading效果

                //this.txtMsg.UseWaitCursor = true;

                var monitorMsg = ServiceFactory.OutPutMonitor();
                SetOutput(monitorMsg);
            }
            catch (Exception ex)
            {
                SetOutput(ex.ToString());
            }
            finally
            {
                this.HideLoadingLayer();//隐藏loading效果

                //this.txtMsg.UseWaitCursor = false;
            }
        }

        #endregion

        #region 按钮事件

        private void btnSetConfig_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrWhiteSpace(this.txtConfigKey.Text.Trim()))
            {
                MessageBox.Show("配置键为空");
                return;
            }

            MonitorConfig.ResetAppSetting(this.txtConfigKey.Text.Trim(), this.txtConfigValue.Text.Trim());

            InitAppSettingConfigInfo();

            MessageBox.Show("配置完成");
        }

        private void btnResetTimer_Click(object sender, EventArgs e)
        {
            InitTimer();
            MessageBox.Show("操作完成");
        }

        private void btnClearMsg_Click(object sender, EventArgs e)
        {
            SetOutput(string.Empty);
        }

        private void btnNotePad_Click(object sender, EventArgs e)
        {
            var msg = this.txtMsg.Text;
            if (string.IsNullOrWhiteSpace(msg) == true)
            {
                MessageBox.Show("没有需要显示的内容");
                return;
            }

            //打开记事本 显示完整信息
            NotePadUtil.OpenNotePad(ProcessWindowStyle.Maximized, msg);
        }

        private void btnRestart_Click(object sender, EventArgs e)
        {
            MonitorConfig.IsStopMonitor = false;
            currentTimer.Tick += currentTimer_Tick;

            this.btnStopMonitor.Enabled = true;
            this.btnRestart.Enabled = false;
        }

        private void btnStopMonitor_Click(object sender, EventArgs e)
        {
            MonitorConfig.IsStopMonitor = true;
            currentTimer.Tick -= currentTimer_Tick;

            this.btnRestart.Enabled = true;
            this.btnStopMonitor.Enabled = false;
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            this.btnStart.Enabled = false;

            var tskFactory = new TaskFactory();
            tskFactory.StartNew(() =>
            {
                //Thread.Sleep(1000);
                KeepWatching(null);
            });

            this.btnStart.Enabled = true;
        }

        private void btnShowMonitorItem_Click(object sender, EventArgs e)
        {
            ShowOrActiveForm<MonitorListForm>("MonitorListForm");
        }

        /// <summary>
        /// 打开form窗口
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="FormName"></param>
        private void ShowOrActiveForm<T>(string FormName) where T : Form, new()
        {
            Form fm = null;

            if (Application.OpenForms[FormName] != null)
            {
                fm = Application.OpenForms[FormName];
                fm.Activate();
            }
            else
            {
                fm = new T();
                fm.Show();
            }
        }

        private void ncSM_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (this.WindowState != FormWindowState.Minimized)
            {
                this.WindowState = FormWindowState.Minimized;
                this.Hide();
            }
            else
            {
                this.Visible = true;
                this.WindowState = FormWindowState.Normal;
                this.Activate();
            }
        }

        private void MainForm_SizeChanged(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.Hide();
            }
        }

        private void tsmShow_Click(object sender, EventArgs e)
        {
            if (this.WindowState != FormWindowState.Minimized)
            {
                return;
            }
            this.Visible = true;
            this.WindowState = FormWindowState.Normal;
            this.Activate();
        }

        private void tsmHide_Click(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                return;
            }
            this.WindowState = FormWindowState.Minimized;
            this.Hide();
        }

        private void tsmExit_Click(object sender, EventArgs e)
        {
            this.Close();
            this.Dispose();
            Application.Exit();
        }

        private void tsmAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Written By JeffWong", "PowerServiceMonitor", MessageBoxButtons.OK);
        }

        #endregion

        #region 退出

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            var result = MessageBox.Show("确定退出吗？", "退出", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);
            if (result == DialogResult.OK)
            {
                Application.ExitThread();
            }
            else
            {
                e.Cancel = true;
            }
        }

        #endregion

        #region 菜单

        private void miModifyPwd_Click(object sender, EventArgs e)
        {
            ShowOrActiveForm<ModifyUserForm>("ModifyUserForm");
        }

        private void miLogAdmin_Click(object sender, EventArgs e)
        {
            ShowOrActiveForm<LogConfigForm>("LogConfigForm");
        }

        private void miQuit_Click(object sender, EventArgs e)
        {
            var result = MessageBox.Show("确定退出吗？", "退出", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);
            if (result == DialogResult.OK)
            {
                Application.ExitThread();
            }
        }

        #endregion

        #region loading效果

        /// <summary>
        /// 显示遮罩层
        /// </summary>
        protected void ShowLoadingLayer()
        {
            SafeOpLoading(true);
        }

        /// <summary>
        /// 隐藏遮罩层
        /// </summary>
        protected void HideLoadingLayer()
        {
            SafeOpLoading(false);
        }

        private void SafeOpLoading(bool isShow)
        {
            if (this.loadingCtrl == null)
            {
                return;
            }
            if (this.loadingCtrl.InvokeRequired == false)
            {
                this.loadingCtrl.Enabled = isShow;
                this.loadingCtrl.Visible = isShow;
            }
            else
            {
                Action<bool> action = SafeOpLoading;
                this.BeginInvoke(action, new object[] { isShow });
            }
        }

        #endregion

        //#region   拦截Windows消息 当点击关闭时，提示是否确认关闭

        //protected override void WndProc(ref   Message m)
        //{
        //    const int WM_SYSCOMMAND = 0x0112;
        //    const int SC_CLOSE = 0xF060;
        //    if (m.Msg == WM_SYSCOMMAND && (int)m.WParam == SC_CLOSE)
        //    {
        //        //捕捉关闭窗体消息      

        //        DialogResult dr = MessageBox.Show("确认推出应用程序吗?", "关闭", MessageBoxButtons.OKCancel);
        //        if (dr == DialogResult.OK)
        //        {
        //            this.Close();
        //            this.Dispose();
        //            Application.Exit();
        //        }
        //        else
        //        {
        //            this.WindowState = FormWindowState.Minimized;
        //            this.Hide();
        //        }
        //    }
        //    base.WndProc(ref   m);
        //}

        //#endregion
    }
}
