﻿using System;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace PowerServiceMonitor
{
    using DotNet.Common.Util;
    using PowerServiceMonitor.Model;
    using PowerServiceMonitor.Util;

    public partial class MonitorListForm : Form
    {
        #region 构造函数

        public MonitorListForm()
        {
            InitializeComponent();

            InitDdlData();

            BindData();
        }

        #endregion

        #region 事件

        private void btnAdd_Click(object sender, EventArgs e)
        {
            try
            {
                var item = new MonitorItemInfo
                {
                    AppType = this.cbAppType.SelectedItem.ToString(),
                    SvcName = this.txtSvcName.Text.Trim(),
                    SvcValue = this.txtSvcValue.Text.Trim(),
                    IsEnabled = this.ckbIsEnabled.Checked,
                    IsSendEmail = this.ckbIsSendEmail.Checked,
                    IsSendSMS = this.ckbIsSendSMS.Checked,
                };

                var bizResult = MonitorItemUtil.AddMonitorItem(MonitorConfig.MonitorConfigFilePath, item);
                if (bizResult.IsOK == false)
                {
                    MessageBox.Show(bizResult.Message);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
            finally
            {
                BindData();
            }
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            BindData();
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            var selectedCollection = this.gvMonitorItem.SelectedRows;
            if (selectedCollection.Count < 1)
            {
                MessageBox.Show("请选择要删除的监控项");
                return;
            }

            try
            {
                foreach (DataGridViewRow row in selectedCollection)
                {
                    var svcName = Convert.ToString(row.Cells["SvcName"].Value);
                    if (string.IsNullOrWhiteSpace(svcName))
                    {
                        continue;
                    }

                    var monitorItem = new MonitorItemInfo
                    {
                        SvcName = svcName,
                    };
                    var bizResult = MonitorItemUtil.DeleteMonitorItem(MonitorConfig.MonitorConfigFilePath, monitorItem);
                    if (bizResult.IsOK == false)
                    {
                        LogServiceBuilder.AppendLog("删除{0}失败，返回信息:{1}", svcName, bizResult.Message);
                    }
                }
            }
            finally
            {
                BindData();
            }

        }

        private void btnDelAll_Click(object sender, EventArgs e)
        {
            var allCollection = this.gvMonitorItem.Rows;
            if (allCollection.Count < 1)
            {
                MessageBox.Show("没有需要删除的监控项");
                return;
            }

            try
            {
                foreach (DataGridViewRow row in allCollection)
                {
                    var svcName = Convert.ToString(row.Cells["SvcName"].Value);
                    if (string.IsNullOrWhiteSpace(svcName))
                    {
                        continue;
                    }

                    var monitorItem = new MonitorItemInfo
                    {
                        SvcName = svcName,
                    };
                    var bizResult = MonitorItemUtil.DeleteMonitorItem(MonitorConfig.MonitorConfigFilePath, monitorItem);
                    if (bizResult.IsOK == false)
                    {
                        LogServiceBuilder.AppendLog("删除{0}失败，返回信息:{1}", svcName, bizResult.Message);
                    }
                }
            }
            finally
            {
                BindData();
            }

        }

        private void btnNotePad_Click(object sender, EventArgs e)
        {
            if (File.Exists(MonitorConfig.MonitorConfigFilePath) == false)
            {
                MessageBox.Show(string.Format("{0}不存在!", MonitorConfig.MonitorConfigFilePath));
                return;
            }
            var msg = File.ReadAllText(MonitorConfig.MonitorConfigFilePath, Encoding.UTF8);

            //打开记事本 显示完整信息
            NotePadUtil.OpenNotePad(ProcessWindowStyle.Maximized, msg);
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            var result = MessageBox.Show("确定退出吗？", "退出", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);
            if (result == DialogResult.OK)
            {
                Application.ExitThread();
            }
        }

        private void btnResetIsEnabled_Click(object sender, EventArgs e)
        {
            var selectedCollection = this.gvMonitorItem.SelectedRows;
            if (selectedCollection.Count < 1)
            {
                MessageBox.Show("请选择要修改启停的监控项");
                return;
            }

            UpdateMonitorItem("IsEnabled");
        }

        private void btnResetAllIsEnabled_Click(object sender, EventArgs e)
        {
            var isEnabled = false;
            if (this.btnResetAllIsEnabled.Text == "全部启用")
            {
                isEnabled = true;
            }

            this.btnResetAllIsEnabled.Enabled = false;

            try
            {
                var allCollection = this.gvMonitorItem.Rows;
                foreach (DataGridViewRow row in allCollection)
                {
                    var svcName = Convert.ToString(row.Cells["SvcName"].Value);
                    if (string.IsNullOrWhiteSpace(svcName))
                    {
                        continue;
                    }

                    var monitorItem = new MonitorItemInfo
                    {
                        SvcName = svcName,

                        AppType = Convert.ToString(row.Cells["AppType"].Value),
                        SvcValue = Convert.ToString(row.Cells["SvcValue"].Value),
                        IsEnabled = isEnabled, //启停参数设置
                        IsSendEmail = Convert.ToBoolean(row.Cells["IsSendEmail"].Value),
                        IsSendSMS = Convert.ToBoolean(row.Cells["IsSendSMS"].Value),
                    };

                    var bizResult = MonitorItemUtil.ModifyMonitorItem(MonitorConfig.MonitorConfigFilePath, monitorItem);
                    if (bizResult.IsOK == false)
                    {
                        LogServiceBuilder.AppendLog("修改{0}失败，返回信息:{1}", svcName, bizResult.Message);
                    }
                }
            }
            finally
            {
                BindData();
            }

            Thread.Sleep(2000);
            this.btnResetAllIsEnabled.Enabled = true;
            this.btnResetAllIsEnabled.Text = isEnabled ? "全部停用" : "全部启用";
        }

        private void btnResetIsSendEmail_Click(object sender, EventArgs e)
        {
            var selectedCollection = this.gvMonitorItem.SelectedRows;
            if (selectedCollection.Count < 1)
            {
                MessageBox.Show("请选择要修改发送邮件提醒的监控项");
                return;
            }

            UpdateMonitorItem("IsSendEmail");
        }

        private void btnResetIsSendSMS_Click(object sender, EventArgs e)
        {
            var selectedCollection = this.gvMonitorItem.SelectedRows;
            if (selectedCollection.Count < 1)
            {
                MessageBox.Show("请选择要修改发送短信提醒的监控项");
                return;
            }

            UpdateMonitorItem("IsSendSMS");
        }

        #endregion

        #region 方法

        private void InitDdlData()
        {
            var appTypes = Enum.GetNames(typeof(AppTypeEnum));
            this.cbAppType.Items.AddRange(appTypes);

            this.cbAppType.SelectedIndex = 0;
        }

        private void BindData()
        {
            try
            {
                var monitorItems = MonitorItemUtil.LoadMonitorItems(MonitorConfig.MonitorConfigFilePath);

                this.gvMonitorItem.AutoGenerateColumns = true;
                this.gvMonitorItem.AllowUserToAddRows = false;

                this.gvMonitorItem.DataSource = monitorItems;

                this.gvMonitorItem.Columns[0].HeaderText = "应用类型";
                this.gvMonitorItem.Columns[0].Width = 100;
                this.gvMonitorItem.Columns[0].ReadOnly = true;

                this.gvMonitorItem.Columns[1].HeaderText = "监控名";
                this.gvMonitorItem.Columns[1].Width = 150;
                this.gvMonitorItem.Columns[1].ReadOnly = true;

                this.gvMonitorItem.Columns[2].HeaderText = "监控值";
                this.gvMonitorItem.Columns[2].Width = 323;

                this.gvMonitorItem.Columns[3].HeaderText = "启用";
                this.gvMonitorItem.Columns[3].Width = 70;

                this.gvMonitorItem.Columns[4].HeaderText = "发送邮件";
                this.gvMonitorItem.Columns[4].Width = 70;

                this.gvMonitorItem.Columns[5].HeaderText = "发送短信";
                this.gvMonitorItem.Columns[5].Width = 70;

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "出错了", MessageBoxButtons.AbortRetryIgnore);
                LogServiceBuilder.AppendLog(ex);
            }
        }

        private void UpdateMonitorItem(string columnName)
        {
            var selectedCollection = this.gvMonitorItem.SelectedRows;
            try
            {
                foreach (DataGridViewRow row in selectedCollection)
                {
                    var svcName = Convert.ToString(row.Cells["SvcName"].Value);
                    if (string.IsNullOrWhiteSpace(svcName))
                    {
                        continue;
                    }

                    var monitorItem = new MonitorItemInfo
                    {
                        SvcName = svcName,

                        AppType = Convert.ToString(row.Cells["AppType"].Value),
                        SvcValue = Convert.ToString(row.Cells["SvcValue"].Value),
                        IsEnabled = Convert.ToBoolean(row.Cells["IsEnabled"].Value),
                        IsSendEmail = Convert.ToBoolean(row.Cells["IsSendEmail"].Value),
                        IsSendSMS = Convert.ToBoolean(row.Cells["IsSendSMS"].Value),
                    };

                    //待设置值取反逻辑控制
                    if (string.Compare(columnName, "IsEnabled", true) == 0)
                    {
                        monitorItem.IsEnabled = !monitorItem.IsEnabled;
                    }
                    if (string.Compare(columnName, "IsSendEmail", true) == 0)
                    {
                        monitorItem.IsSendEmail = !monitorItem.IsSendEmail;
                    }
                    if (string.Compare(columnName, "IsSendSMS", true) == 0)
                    {
                        monitorItem.IsSendSMS = !monitorItem.IsSendSMS;
                    }

                    var bizResult = MonitorItemUtil.ModifyMonitorItem(MonitorConfig.MonitorConfigFilePath, monitorItem);
                    if (bizResult.IsOK == false)
                    {
                        LogServiceBuilder.AppendLog("修改{0}失败，返回信息:{1}", svcName, bizResult.Message);
                    }
                }
            }
            finally
            {
                BindData();
            }

        }

        #endregion

    }
}
