﻿using System;
using System.ServiceProcess;
using System.Threading;

namespace ServiceMonitor
{
    using DotNet.Common.Util;
    using PowerServiceMonitor;
    using PowerServiceMonitor.Util;

    public partial class MonitorService : ServiceBase
    {
        #region fields

        private static Thread trdWorker = null;

        private static Timer currentTimer = null;

        #endregion

        #region events

        /// <summary>
        /// 初始化对象
        /// </summary>
        public MonitorService()
        {
            //InitializeComponent();
            this.ServiceName = "MonitorService";
            //CanPauseAndContinue = true;
            CanHandleSessionChangeEvent = true;
            CanShutdown = true;
            CanStop = true;
        }

        /// <summary>
        /// 启动服务
        /// </summary>
        /// <param name="args"></param>
        protected override void OnStart(string[] args)
        {
            LogServiceBuilder.AppendLog("守护服务MonitorService启动");

            trdWorker = new Thread(new ThreadStart(InitTimer));
            trdWorker.Name = "Power_MonitorService_Thread";
            trdWorker.IsBackground = true;
            trdWorker.Start();
        }

        public static void InitTimer()
        {
            int interval = 1000 * 60 * MonitorConfig.GetIntervalMinutes();//每隔多少分钟监控一次
            currentTimer = new Timer(KeepWatching, null, interval, Timeout.Infinite);
        }

        /// <summary>
        /// 终止服务
        /// </summary>
        protected override void OnStop()
        {
            try
            {
                if (trdWorker == null)
                {
                    return;
                }

                trdWorker.Abort();
                LogServiceBuilder.AppendLog(string.Format("{0}线程已经终止", trdWorker.Name));

            }
            finally
            {
                base.OnStop();
            }
        }

        #endregion

        #region methods

        /// <summary>
        /// 保持监控 防止服务意外停止
        /// </summary>
        private static void KeepWatching(object state)
        {
            try
            {
                var monitorMsg = ServiceFactory.OutPutMonitor();
                LogServiceBuilder.AppendLog(monitorMsg);
            }
            catch (Exception ex)
            {
                LogServiceBuilder.AppendLog(ex);
            }
        }

        #endregion

    }
}
