﻿using System;
using System.Linq;

namespace PowerServiceMonitor.Service
{
    using DotNet.Common.Util;
    using PowerServiceMonitor.Model;
    using PowerServiceMonitor.Util;

    /// <summary>
    /// 管理员服务 TODO
    /// </summary>
    public class AdminUserService
    {
        #region 方法

        /// <summary>
        /// 登录
        /// </summary>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public static PowerBizResult<string> UserLogin(AdminUserInfo userInfo)
        {

            var resultInfo = new PowerBizResult<string>(true);

            try
            {   //1、验证
                resultInfo = CheckLoginUser(userInfo);
                if (resultInfo.IsOK == false)
                {
                    return resultInfo;
                }

                //2、调用远程登录服务 todo
                userInfo.UserName = userInfo.UserName.Trim();
                userInfo.Pwd = userInfo.Pwd.Trim();
                userInfo.EncryptPwd = GetHashPwd(userInfo.Pwd);

                //演示用 简单验证登录逻辑
                var curUser = GetCurAdminUser();
                var tmpUserName = curUser.UserName; //"admin";
                var tmpHashPwd = curUser.EncryptPwd; //"4cb6046f9216f8a54979e80b2a44954ec159df40f252ea2e94df234f1715d08e";

                if (string.Compare(userInfo.UserName, tmpUserName, true) != 0)
                {
                    resultInfo.IsOK = false;
                    resultInfo.Message = "用户名不正确";
                }
                else if (string.Compare(userInfo.EncryptPwd, tmpHashPwd, true) != 0)
                {
                    resultInfo.IsOK = false;
                    resultInfo.Message = "密码不正确";
                }
            }
            finally
            {
                var loginUser = (userInfo == null) ? string.Empty : userInfo.UserName;
                LogServiceBuilder.AppendLog("用户{0}登录，结果:{1},信息:{2}",
                    loginUser, resultInfo.IsOK, resultInfo.Message);
            }

            return resultInfo;
        }

        /// <summary>
        /// 获取当前用户信息
        /// </summary>
        /// <returns></returns>
        public static AdminUserInfo GetCurAdminUser()
        {
            var listUser = XmlHelper<AdminUserInfo>.GetObject(MonitorConfig.AdminUserConfigFilePath,
                "AdminUserInfos", "AdminUserInfo");
            if (listUser.Count > 1)
            {
                throw new SystemException("当前系统管理员配置有错误");
            }

            var userInfo = listUser.FirstOrDefault();

            return userInfo;
        }

        /// <summary>
        /// 修改密码
        /// </summary>
        /// <param name="userInfo"></param>
        /// <returns></returns>
        public static PowerBizResult<string> ModifyPwd(ModifyUserInfoViewModel userInfo)
        {
            var resultInfo = new PowerBizResult<string>(true);
            AdminUserInfo curUserInfo = null;

            try
            { //1、验证
                resultInfo = CheckUpdateUser(userInfo);
                if (resultInfo.IsOK == false)
                {
                    return resultInfo;
                }

                curUserInfo = GetCurAdminUser();
                var encryptPwd = GetHashPwd(userInfo.OldPwd);

                if (string.Compare(curUserInfo.EncryptPwd, encryptPwd, false) != 0)
                {
                    resultInfo.IsOK = false;
                    resultInfo.Message = "旧密码输入不正确";
                    return resultInfo;
                }

                //2、调用远程改密服务 todo

                //演示用 简单改密码逻辑
                var modifyUserInfo = new AdminUserInfo
                {
                    UserName = curUserInfo.UserName,
                    Pwd = string.Empty,
                    EncryptPwd = GetHashPwd(userInfo.NewPwd),//设置新密码
                };

                var isOK = XmlHelper<AdminUserInfo>.ModifyOneT(MonitorConfig.AdminUserConfigFilePath
                    , "AdminUserInfos", "AdminUserInfo", "UserName", modifyUserInfo.UserName, modifyUserInfo);

                if (isOK == false)
                {
                    resultInfo.IsOK = false;
                    resultInfo.Message = "改密失败";
                }
            }
            finally
            {
                var loginUser = (curUserInfo == null) ? string.Empty : curUserInfo.UserName;
                LogServiceBuilder.AppendLog("当前用户{0}修改密码，结果:{1},信息:{2}",
                    loginUser, resultInfo.IsOK, resultInfo.Message);
            }

            return resultInfo;
        }

        #endregion

        #region 辅助方法

        private static PowerBizResult<string> CheckLoginUser(AdminUserInfo userInfo)
        {
            var resultInfo = new PowerBizResult<string>(true);
            if (userInfo == null)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "用户信息为空";
            }
            else if (string.IsNullOrWhiteSpace(userInfo.UserName))
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "用户名为空";
            }
            else if (string.IsNullOrWhiteSpace(userInfo.Pwd))
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "密码为空";
            }
            return resultInfo;
        }

        private static PowerBizResult<string> CheckUpdateUser(ModifyUserInfoViewModel userInfo)
        {
            var resultInfo = new PowerBizResult<string>(true);
            if (userInfo == null)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "信息为空";
            }
            else if (string.IsNullOrWhiteSpace(userInfo.OldPwd))
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "原密码为空";
            }
            else if (string.IsNullOrWhiteSpace(userInfo.NewPwd))
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "新密码为空";
            }
            else if (string.Compare(userInfo.NewPwd, userInfo.ConfirmedNewPwd, false) != 0)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "两次新密码输入不一致";
            }
            return resultInfo;
        }

        /// <summary>
        /// 获取哈希后的密码
        /// </summary>
        /// <param name="pwd"></param>
        /// <returns></returns>
        private static string GetHashPwd(string pwd)
        {
            return SecurityUtil.ComputeSHA256Hash(pwd, "PowerServiceMonitor");
        }

        #endregion
    }
}
