﻿using System;
using System.Configuration;
using System.IO;
using System.Text;

namespace PowerServiceMonitor.Util
{
    using DotNet.Common.Util;

    /// <summary>
    /// 服务监控配置相关类
    /// </summary>
    internal class MonitorConfig
    {
        #region 字段

        /// <summary>
        /// 待监控的配置文件路径
        /// </summary>
        public static string MonitorConfigFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory
            , "Config/MonitorServices.config");

        /// <summary>
        /// 管理员配置文件路径
        /// </summary>
        public static string AdminUserConfigFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory
            , "Config/AdminUsers.config");

        /// <summary>
        /// 是否停止监控  监控开关用
        /// </summary>
        public static bool IsStopMonitor = false;

        /// <summary>
        /// 间隔的时间 单位：分钟
        /// </summary>
        public static readonly string MonitorInterval = "MonitorInterval";

        /// <summary>
        /// 日志提供者程序类型 多个以逗号分隔
        /// </summary>
        public static readonly string LogProviderType = "LogProviderType";

        /// <summary>
        /// 应用类型
        /// </summary>
        public static readonly string AppType = "AppType";

        /// <summary>
        /// 服务名
        /// </summary>
        public static readonly string SvcName = "SvcName";

        /// <summary>
        /// 服务对应项
        /// </summary>
        public static readonly string SvcValue = "SvcValue";

        /// <summary>
        /// 是否启用
        /// </summary>
        public static readonly string IsEnabled = "IsEnabled";

        /// <summary>
        /// 是否发送邮件提醒
        /// </summary>
        public static readonly string IsSendEmail = "IsSendEmail";

        /// <summary>
        /// 是否发送短信提醒
        /// </summary>
        public static readonly string IsSendSMS = "IsSendSMS";

        #endregion

        #region 方法

        /// <summary>
        /// 获取服务监控时间间隔
        /// </summary>
        /// <returns></returns>
        public static int GetIntervalMinutes()
        {
            var minutes = 10; //默认10分钟监控一次
            try
            {
                ConfigurationManager.RefreshSection("appSettings");

                minutes = Math.Abs(int.Parse(ConfigurationManager.AppSettings[MonitorConfig.MonitorInterval]));
            }
            catch
            {
                minutes = 10;
            }
            if (minutes < 1)
            {
                minutes = 10;
            }

            return minutes;
        }

        /// <summary>
        /// 获取日志提供者
        /// </summary>
        /// <returns></returns>
        public static string GetLogProviderTypes()
        {
            var result = string.Empty;
            try
            {
                ConfigurationManager.RefreshSection("appSettings");

                result = ConfigurationManager.AppSettings[MonitorConfig.LogProviderType];
            }
            catch
            {

            }

            return result;
        }

        /// <summary>
        /// 获取日志提供者
        /// </summary>
        /// <returns></returns>
        public static string[] GetLogProviderTypeArr()
        {
            var logTypes = MonitorConfig.GetLogProviderTypes() ?? string.Empty;

            var logTypeArr = logTypes.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);

            return logTypeArr;
        }

        /// <summary>
        /// 修改AppSettings配置节点
        /// </summary>
        /// <param name="key"></param>
        /// <param name="newValue"></param>
        public static void ResetAppSetting(string key, string newValue)
        {
            try
            {
                PowerConfigUtil.ResetAppSetting(key, newValue);
            }
            catch (Exception ex)
            {
                LogServiceBuilder.AppendLog("更新AppSetttings节点{0}发生异常：{1}", key, ex);
            }
        }

        /// <summary>
        /// 初始化监控项、管理用户信息配置文件
        /// </summary>
        public static void InitMonitorConfig()
        {
            if (File.Exists(MonitorConfigFilePath) == false)
            {
                var xml = string.Format("<?xml version=\"1.0\" encoding=\"utf-8\"?>{0}<MonitorItemInfos>{0}</MonitorItemInfos>",
                    Environment.NewLine);

                File.WriteAllText(MonitorConfigFilePath, xml, Encoding.UTF8);
            }

            //初始化用户名(admin)和密码(123456)
            if (File.Exists(AdminUserConfigFilePath) == false)
            {
                var xml = string.Format("<?xml version=\"1.0\" encoding=\"utf-8\"?>{0}<AdminUserInfos>{0}   <AdminUserInfo UserName=\"admin\" Pwd=\"\" EncryptPwd=\"4cb6046f9216f8a54979e80b2a44954ec159df40f252ea2e94df234f1715d08e\"/>{0}</AdminUserInfos>",
                    Environment.NewLine);

                File.WriteAllText(AdminUserConfigFilePath, xml, Encoding.UTF8);
            }
        }

        #endregion
    }
}
