﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Xml;

namespace PowerServiceMonitor.Util
{
    using DotNet.Common.Util;
    using PowerServiceMonitor.Model;

    /// <summary>
    /// 监控项帮助
    /// </summary>
    internal class MonitorItemUtil
    {

        /// <summary>
        /// 加载所有代监控的项集合
        /// </summary>
        /// <param name="configFilePath">待监控项配置文件 如：F:\PowerServiceMonitor\Config\MonitorServices.config</param>
        /// <returns></returns>
        public static IList<MonitorItemInfo> LoadMonitorItems(string configFilePath)
        {
            var listResult = XmlHelper<MonitorItemInfo>.GetObject(configFilePath, "MonitorItemInfos", "MonitorItemInfo");

            listResult = listResult.Where(x => EnumHelper.ExistEnum<AppTypeEnum>(x.AppType)).ToList();

            return listResult;
        }

        /// <summary>
        /// 添加监控项至配置文件中
        /// </summary>
        /// <param name="configFilePath"></param>
        /// <param name="item"></param>
        /// <returns></returns>
        public static PowerBizResult<string> AddMonitorItem(string configFilePath, MonitorItemInfo item)
        {
            var resultInfo = new PowerBizResult<string>(true);
            if (item == null)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "监控项为空";
            }
            else if (string.IsNullOrWhiteSpace(item.AppType) == true)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "监控应用类型为空";
            }
            else if (Enum.GetNames(typeof(AppTypeEnum)).Where(x => string.Compare(x, item.AppType, true) == 0).Any() == false)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = string.Format("暂不支持的监控应用类型：{0}", item.AppType);
            }
            else if (string.IsNullOrWhiteSpace(item.SvcName) == true)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "监控键为空";
            }
            else if (string.IsNullOrWhiteSpace(item.SvcValue) == true)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "监控值为空";
            }
            if (resultInfo.IsOK == false)
            {
                return resultInfo;
            }

            //添加监控项至config文件
            var isOK = XmlHelper<MonitorItemInfo>.InsertOneT(configFilePath, "MonitorItemInfos", "MonitorItemInfo", "SvcName",
                  item.SvcName, item);

            if (isOK == false)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "添加失败，可能键值重复";
            }

            return resultInfo;
        }

        /// <summary>
        /// 删除监控项
        /// </summary>
        /// <param name="configFilePath"></param>
        /// <param name="item"></param>
        /// <returns></returns>
        public static PowerBizResult<string> DeleteMonitorItem(string configFilePath, MonitorItemInfo item)
        {
            var resultInfo = new PowerBizResult<string>(true);
            if (item == null)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "监控项为空";
            }
            else if (string.IsNullOrWhiteSpace(item.SvcName) == true)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "监控键为空";
            }
            if (resultInfo.IsOK == false)
            {
                return resultInfo;
            }

            //添加监控项至config文件
            var isOK = XmlHelper<MonitorItemInfo>.DeleteOneT(configFilePath, "MonitorItemInfos", "MonitorItemInfo",
                "SvcName", item.SvcName);

            if (isOK == false)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "删除失败";
            }

            return resultInfo;
        }

        /// <summary>
        /// 修改监控项
        /// </summary>
        /// <param name="configFilePath"></param>
        /// <param name="item"></param>
        /// <returns></returns>
        public static PowerBizResult<string> ModifyMonitorItem(string configFilePath, MonitorItemInfo item)
        {
            var resultInfo = new PowerBizResult<string>(true);
            if (item == null)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "监控项为空";
            }
            else if (string.IsNullOrWhiteSpace(item.SvcName) == true)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "监控键为空";
            }
            if (resultInfo.IsOK == false)
            {
                return resultInfo;
            }

            //添加监控项至config文件
            var isOK = XmlHelper<MonitorItemInfo>.ModifyOneT(configFilePath, "MonitorItemInfos", "MonitorItemInfo",
                "SvcName", item.SvcName, item);

            if (isOK == false)
            {
                resultInfo.IsOK = false;
                resultInfo.Message = "删除失败";
            }

            return resultInfo;
        }
    }
}