﻿using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace PowerServiceMonitor.WinControl
{
    /// <summary>
    /// loading效果控件
    /// </summary>
    [ToolboxBitmap(typeof(PowerLoadingLayer))]
    public class PowerLoadingLayer : System.Windows.Forms.Control
    {

        #region 字段

        private bool _transparentBG = true;

        private int _alpha = 125;

        private Container components = new Container();

        #endregion

        #region 属性

        [Category("PowerLoadingLayer"), Description("是否使用透明,默认为True")]
        public bool TransparentBG
        {
            get { return _transparentBG; }
            set
            {
                _transparentBG = value;
                this.Invalidate();
            }
        }

        [Category("PowerLoadingLayer"), Description("设置透明度")]
        public int Alpha
        {
            get { return _alpha; }
            set
            {
                _alpha = value;
                this.Invalidate();
            }
        }

        #endregion

        #region 构造函数

        public PowerLoadingLayer()
            : this(125, true)
        {
        }

        public PowerLoadingLayer(int alpha, bool isShowLoading)
        {
            SetStyle(ControlStyles.Opaque, true);

            base.CreateControl();

            this._alpha = alpha;

            if (isShowLoading == false)
            {
                return;
            }

            var pictureBox = new PictureBox();
            pictureBox.BackColor = System.Drawing.Color.White;
            pictureBox.Image = global::PowerServiceMonitor.Properties.Resources.loading;
            pictureBox.Name = "PictureBox_Loading";
            pictureBox.Size = new System.Drawing.Size(48, 48);
            pictureBox.SizeMode = System.Windows.Forms.PictureBoxSizeMode.AutoSize;
            Point Location = new Point(this.Location.X + (this.Width - pictureBox.Width) / 2, this.Location.Y + (this.Height - pictureBox.Height) / 2);
            pictureBox.Location = Location;
            pictureBox.Anchor = AnchorStyles.None;
            this.Controls.Add(pictureBox);
        }

        #endregion

        #region 方法

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (!((components == null)))
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        /// <summary>
        /// 自定义绘制窗体
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
        {
            float vlblControlWidth;
            float vlblControlHeight;

            Pen labelBorderPen;
            SolidBrush labelBackColorBrush;

            if (_transparentBG)
            {
                Color drawColor = Color.FromArgb(this._alpha, this.BackColor);
                labelBorderPen = new Pen(drawColor, 0);
                labelBackColorBrush = new SolidBrush(drawColor);
            }
            else
            {
                labelBorderPen = new Pen(this.BackColor, 0);
                labelBackColorBrush = new SolidBrush(this.BackColor);
            }
            base.OnPaint(e);
            vlblControlWidth = this.Size.Width;
            vlblControlHeight = this.Size.Height;
            e.Graphics.DrawRectangle(labelBorderPen, 0, 0, vlblControlWidth, vlblControlHeight);
            e.Graphics.FillRectangle(labelBackColorBrush, 0, 0, vlblControlWidth, vlblControlHeight);

        }

        protected override CreateParams CreateParams
        {
            get
            {
                CreateParams cp = base.CreateParams;
                cp.ExStyle |= 0x20;  // 开启 WS_EX_TRANSPARENT,使控件支持透明
                return cp;
            }
        }

        #endregion

    }
}