﻿using System;
using System.IO;
using System.ServiceProcess;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Generic;

namespace WinServiceDynamicInstallTool
{
    using DotNet.Common.WinSvcInstaller;

    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
            Init();
        }

        /// <summary>
        /// 初始化一些数据
        /// </summary>
        private void Init()
        {
            //this.cbStartType.Items.Add("Automatic");
            //this.cbStartType.Items.Add("Manual");
            //this.cbStartType.Items.Add("Disabled");
            this.cbStartType.SelectedIndex = 0;

            //this.cbAccount.Items.Add("LocalSystem");
            //this.cbAccount.Items.Add("User");
            //this.cbAccount.Items.Add("LocalService");
            //this.cbAccount.Items.Add("NetworkService");
            this.cbAccount.SelectedIndex = 0;
        }

        #region fields

        private WindowsServiceInstallInfo wsInstallInfo = null;
        private WindowsServiceInstallUtil wsInstallUtil = null;
        private bool isSucceed = false;

        #endregion

        #region install

        private void btnInstall_Click(object sender, EventArgs e)
        {
            Install();
        }

        /// <summary>
        /// 安装
        /// </summary>
        private void Install()
        {
            string fileName = this.txtWinSvcPath.Text.Trim();
            if (string.IsNullOrEmpty(fileName) == true)
            {
                MessageBox.Show("请选择需要安装的windows服务源.exe文件");
                return;
            }
            FileInfo fileInfo = new FileInfo(fileName);

            string svcName = this.txtSvcName.Text.Trim();//服务名
            if (string.IsNullOrEmpty(svcName) == true)
            {
                svcName = fileInfo.Name.Substring(0, fileInfo.Name.Length - fileInfo.Extension.Length);//默认的服务名
            }
            string svcDisplayName = this.txtDisplayName.Text.Trim();//服务显示名称
            if (string.IsNullOrEmpty(svcDisplayName) == true)
            {
                svcDisplayName = svcName;
            }
            string description = this.rtbDesc.Text.Trim();//服务简介
            string startType = this.cbStartType.Items[this.cbStartType.SelectedIndex].ToString().Trim();//选择启动类型
            WindowsServiceAccountType accountType = WindowsServiceAccountType.LocalSystem;
            string account = this.cbAccount.Items[this.cbAccount.SelectedIndex].ToString().Trim();//选择账户类型
            switch (account)
            {
                default:
                case "LocalSystem":
                    accountType = WindowsServiceAccountType.LocalSystem;
                    break;
                case "User":
                    accountType = WindowsServiceAccountType.User;
                    break;
                case "LocalService":
                    accountType = WindowsServiceAccountType.LocalService;
                    break;
                case "NetworkService":
                    accountType = WindowsServiceAccountType.NetworkService;
                    break;
            }
            if (IsServiceExists(svcName)==true)
            {
                MessageBox.Show(string.Format("{0}服务名已经存在.", svcName));
                return;
            }
            string userName = this.txtUserName.Text.Trim();
            string pwd = this.txtPwd.Text.Trim();
            if (string.IsNullOrEmpty(userName) == true && string.IsNullOrEmpty(pwd) == true)
            {
                wsInstallInfo = new WindowsServiceInstallInfo(svcName, svcDisplayName, description, fileInfo.DirectoryName, fileInfo.Name, startType);
            }
            else
            {
                wsInstallInfo = new WindowsServiceInstallInfo(svcName, svcDisplayName, svcDisplayName, fileInfo.DirectoryName, fileInfo.Name, accountType, userName, pwd, startType);
            }
            wsInstallUtil = new WindowsServiceInstallUtil(wsInstallInfo);
            isSucceed = wsInstallUtil.Install();
            MessageBox.Show(string.Format("{0}服务安装{1}! ", svcName, isSucceed ? "成功" : "失败"));
        }

        #endregion

        #region uninstall

        private void btnUninstall_Click(object sender, EventArgs e)
        {
            UnInstall();
        }

        /// <summary>
        /// 卸载
        /// </summary>
        private void UnInstall()
        {
            string svcName = this.txtUSvcName.Text.Trim();
            if (string.IsNullOrEmpty(svcName) == true)
            {
                this.lblUWarn.Visible = true;
                return;
            }
            if (IsServiceExists(svcName) == false)
            {
                MessageBox.Show(string.Format("{0}服务名不存在，无法卸载", svcName));
                return;
            }

            this.lblUWarn.Visible = false;
            string fileName = this.txtUWinSrv.Text.Trim();
            if (string.IsNullOrEmpty(fileName) == true)
            {
                MessageBox.Show("请选择要卸载服务源文件所在路径");
                return;
            }
            FileInfo fileInfo = new FileInfo(fileName);
            wsInstallInfo = new WindowsServiceInstallInfo(svcName, fileInfo.DirectoryName, fileInfo.Name);
            wsInstallUtil = new WindowsServiceInstallUtil(wsInstallInfo);
            try
            {
                ServiceController control = new ServiceController(svcName);
                if (control != null)
                {
                    isSucceed = wsInstallUtil.Uninstall();
                    MessageBox.Show(string.Format("{0}服务卸载{1}! ", svcName, isSucceed ? "成功" : "失败"));
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("{0}服务卸载时发生异常：{1}", svcName, ex.Message));
            }
        }

        #endregion

        #region service state

        private void txtStateSvcName_MouseLeave(object sender, EventArgs e)
        {
            SetServiceState();
        }

        /// <summary>
        /// 服务启动
        /// </summary>
        private void btnStart_Click(object sender, EventArgs e)
        {
            string svcName = this.txtStateSvcName.Text.Trim();
            if (IsServiceExists(svcName) == false)
            {
                return;
            }
            this.gbState.Enabled = false;
            this.gbState.UseWaitCursor = true;
            ServiceController ctrl = new ServiceController(svcName);
            ctrl.Start();
            ctrl.WaitForStatus(ServiceControllerStatus.Running);
            SetServiceState();
            this.gbState.Enabled = true;
            this.gbState.UseWaitCursor = false;
        }

        /// <summary>
        /// 停止
        /// </summary>
        private void btnStop_Click(object sender, EventArgs e)
        {
            string svcName = this.txtStateSvcName.Text.Trim();
            if (IsServiceExists(svcName) == false)
            {
                return;
            }
            this.gbState.Enabled = false;
            this.gbState.UseWaitCursor = true;
            ServiceController ctrl = new ServiceController(svcName);
            ctrl.Stop();
            ctrl.WaitForStatus(ServiceControllerStatus.Stopped);
            SetServiceState();
            this.gbState.Enabled = true;
            this.gbState.UseWaitCursor = false;
        }

        /// <summary>
        /// 暂停
        /// </summary>
        private void btnPause_Click(object sender, EventArgs e)
        {
            string svcName = this.txtStateSvcName.Text.Trim();
            if (IsServiceExists(svcName) == false)
            {
                return;
            }
            this.gbState.Enabled = false;
            this.gbState.UseWaitCursor = true;
            ServiceController ctrl = new ServiceController(svcName);
            ctrl.Pause();
            ctrl.WaitForStatus(ServiceControllerStatus.Paused);
            SetServiceState();
            this.gbState.Enabled = true;
            this.gbState.UseWaitCursor = false;
        }

        /// <summary>
        /// 恢复
        /// </summary>
        private void btnContinue_Click(object sender, EventArgs e)
        {
            string svcName = this.txtStateSvcName.Text.Trim();
            if (IsServiceExists(svcName) == false)
            {
                return;
            }
            this.gbState.Enabled = false;
            this.gbState.UseWaitCursor = true;
            ServiceController ctrl = new ServiceController(svcName);
            ctrl.Continue();
            ctrl.WaitForStatus(ServiceControllerStatus.Running);
            SetServiceState();
            this.gbState.Enabled = true;
            this.gbState.UseWaitCursor = false;
        }

        /// <summary>
        /// 获取当前服务的状态，并设置相关按钮启用
        /// </summary>
        private void SetServiceState()
        {
            bool isCanStart = false;
            bool isCanStop = false;
            bool isCanPause = false;
            bool isCanContinue = false;
            string svcName = this.txtStateSvcName.Text.Trim();
            if (IsServiceExists(svcName) == false)
            {
                SetBtnEnabled(isCanStart, isCanStop, isCanPause, isCanContinue);
                return;
            }
            ServiceController ctrl = new ServiceController(svcName);
            switch (ctrl.Status) //当前状态
            {
                default:
                    isCanStart = true;
                    break;

                case ServiceControllerStatus.StopPending:
                case ServiceControllerStatus.Stopped: //已经停止
                    isCanStart = true;
                    isCanStop = false;
                    isCanPause = false;
                    isCanContinue = false;
                    break;

                case ServiceControllerStatus.StartPending:
                case ServiceControllerStatus.Running: //正在运行
                    isCanStart = false;
                    isCanStop = ctrl.CanStop;
                    isCanPause = ctrl.CanPauseAndContinue;
                    isCanContinue = false;
                    break;

                case ServiceControllerStatus.PausePending:
                case ServiceControllerStatus.Paused: //已经暂停
                    isCanStart = false;
                    isCanStop = ctrl.CanStop;
                    isCanPause = false;
                    isCanContinue = ctrl.CanPauseAndContinue;
                    break;

                case ServiceControllerStatus.ContinuePending://继续
                    isCanStart = false;
                    isCanStop = ctrl.CanStop;
                    isCanPause = ctrl.CanPauseAndContinue;
                    isCanContinue = false;
                    break;
            }
            SetBtnEnabled(isCanStart, isCanStop, isCanPause, isCanContinue);
        }

        private void SetBtnEnabled(bool isCanStart, bool isCanStop, bool isCanPause, bool isCanContinue)
        {
            this.btnStart.Enabled = isCanStart;
            this.btnStop.Enabled = isCanStop;
            this.btnPause.Enabled = isCanPause;
            this.btnContinue.Enabled = isCanContinue;
        }

        #endregion

        #region double click

        private void txtWinSvcPath_DoubleClick(object sender, System.EventArgs e)
        {
            ofdWinService.Title = "打开(Open)";
            //ofdWinService.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);//为了获取特定的系统文件夹，可以使用System.Environment类的静态方法GetFolderPath()。该方法接受一个Environment.SpecialFolder枚举，其中可以定义要返回路径的哪个系统目录
            ofdWinService.Filter = "windows服务(*.exe)|*.exe";
            if (ofdWinService.ShowDialog() == DialogResult.OK)
            {
                this.txtWinSvcPath.Text = ofdWinService.FileName;
            }
        }

        private void txtUWinSrv_DoubleClick(object sender, EventArgs e)
        {
            ofdUWinService.Title = "打开(Open)";
            //ofdUWinService.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);//为了获取特定的系统文件夹，可以使用System.Environment类的静态方法GetFolderPath()。该方法接受一个Environment.SpecialFolder枚举，其中可以定义要返回路径的哪个系统目录
            ofdUWinService.Filter = "windows服务(*.exe)|*.exe";
            if (ofdUWinService.ShowDialog() == DialogResult.OK)
            {
                this.txtUWinSrv.Text = ofdUWinService.FileName;
            }
        }

        #endregion

        #region util

        /// <summary>
        /// 服务是否存在
        /// </summary>
        /// <param name="svcName"></param>
        /// <returns></returns>
        private bool IsServiceExists(string svcName)
        {
            if (string.IsNullOrEmpty(svcName) == true)
            {
                return false;
            }
            bool flag = false;
            //ServiceController[] ctrllers = ServiceController.GetServices();
            //foreach (ServiceController item in ctrllers) //通过遍历查找服务名是否存在 可以实现功能 但效率可能不高
            //{
            //    if (string.Compare(svcName.Trim(),item.ServiceName)==0)
            //    {
            //        flag = true;
            //        break;
            //    }
            //}

            try
            {
                ServiceController control = new ServiceController(svcName);
                ServiceControllerStatus status = control.Status;//这一行必不可少
                if (control != null)
                {
                    //if (control.ServiceType == ServiceType.Win32OwnProcess)//判断服务类型
                    //{
                    flag = true;
                    //}
                }
            }
            catch
            {
                flag = false;
            }
            return flag;
        }

        #endregion

    }
}